#include <Application.h>
#include <Window.h>
#include <View.h>
#include <Shape.h>
#include <Rect.h>
#include <stdio.h>
#include <math.h>

class StateGroup
{
private:
	BView *fView;
public:
	StateGroup(BView *view): fView(view) {fView->PushState();}
	~StateGroup() {fView->PopState();}
};

class LayerGroup
{
private:
	BView *fView;
public:
	LayerGroup(BView *view, uint8 opacity): fView(view) {fView->BeginLayer(opacity);}
	~LayerGroup() {fView->EndLayer();}
};

void GetRoundRectShape(BShape &sh, BRect rc, float r)
{
	sh.MoveTo(rc.RightTop() - BPoint(r, 0));
	sh.ArcTo(r, r, M_PI/2, false, false, rc.RightTop() + BPoint(0, r));
	sh.LineTo(rc.RightBottom() - BPoint(0, r));
	sh.ArcTo(r, r, M_PI/2, false, false, rc.RightBottom() - BPoint(r, 0));
	sh.LineTo(rc.LeftBottom() + BPoint(r, 0));
	sh.ArcTo(r, r, M_PI/2, false, false, rc.LeftBottom() - BPoint(0, r));
	sh.LineTo(rc.LeftTop() + BPoint(0, r));
	sh.ArcTo(r, r, M_PI/2, false, false, rc.LeftTop() + BPoint(r, 0));
	sh.Close();
}

class TestView: public BView
{
private:
	int32 fMode;
	BPoint fPos;
public:
	TestView(BRect frame, const char *name): BView(frame, name, B_FOLLOW_NONE, B_FULL_UPDATE_ON_RESIZE | B_WILL_DRAW | B_SUBPIXEL_PRECISE),
		fMode(0),
		fPos(0, 0)
	{
	}

	void Draw(BRect dirty)
	{
		switch (fMode) {
		case 0: {
			{StateGroup sg(this);
				TranslateBy(fPos.x, fPos.y);
				BShape sh;
				GetRoundRectShape(sh, BRect(0, 0, 63, 63), 8);
				FillShape(&sh);
			}
			break;
		}
		case 1: {
			{StateGroup sg(this);
				TranslateBy(fPos.x, fPos.y);
				BShape sh;
				GetRoundRectShape(sh, BRect(0, 0, 63, 63), 8);
				ClipToShape(&sh);
				FillRect(BRect(0, 0, 63, 63));
			}
			break;
		}
		}
	}

	void KeyDown(const char* bytes, int32 numBytes)
	{
		switch (bytes[0]) {
		case B_LEFT_ARROW: fPos.x--; Invalidate(); break;
		case B_RIGHT_ARROW: fPos.x++; Invalidate(); break;
		case B_UP_ARROW: fPos.y--; Invalidate(); break;
		case B_DOWN_ARROW: fPos.y++; Invalidate(); break;
		case '1': fMode = 0; Invalidate(); break;
		case '2': fMode = 1; Invalidate(); break;
		}
	}
};

class TestWindow: public BWindow
{
private:
	TestView *fView;
public:
	TestWindow(BRect frame): BWindow(frame, "TestApp", B_TITLED_WINDOW, B_ASYNCHRONOUS_CONTROLS)
	{
		fView = new TestView(frame.OffsetToCopy(B_ORIGIN), "view");
		fView->SetResizingMode(B_FOLLOW_ALL);
		AddChild(fView, NULL);
		fView->MakeFocus(true);
	}

	void Quit()
	{
		be_app_messenger.SendMessage(B_QUIT_REQUESTED);
		BWindow::Quit();
	}

};

class TestApplication: public BApplication
{
private:
	TestWindow *fWnd;
public:
	TestApplication(): BApplication("application/x-vnd.test-app")
	{
	}

	void ReadyToRun() {
		fWnd = new TestWindow(BRect(0, 0, 255, 255).OffsetByCopy(64, 64));
		fWnd->Show();
	}
};


int main()
{
	TestApplication app;
	app.Run();
	return 0;
}
